/*

    GlobalSat DG-100 GPS data logger control program.

    Copyright (C) 2009  Paul Fleischer, paul@xpg.dk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

 */
#include "dg100-comm.h"
#include <string.h>


#define MAX_DEVICE_NAME_LENGTH 60
#define MAX_ACTIONS 10

/*#define MODE_NONE 0
#define MODE_LIST 1
#define MODE_SET 2
#define MODE_ENABLE_GMOUSE 3
#define MODE_DISABLE_GMOUSE 4
#define MODE_SET_DEFAULT 5*/

struct action {

};


void print_usage(char *prog_name);
void print_config(struct dg100config *config);
bool parse_mode_set_line(char *arg, struct dg100config *config);
void reset_configuration(struct dg100config *config);

int main(int argc, char *argv[]) {
  int opt;
  char device[MAX_DEVICE_NAME_LENGTH] = "/dev/ttyUSB0";
  const char optstring[] = "l:w:hm:xd:ps:t:T:";
  struct dg100config config;
  bool update_config = false;

  if( argc < 2 ) {
    print_usage(argv[0]);
    exit(-1);
  }

  if( strcmp(argv[1], "-d") == 0 ) {
    strncpy(device, argv[2], MAX_DEVICE_NAME_LENGTH);
  } else if( strcmp(argv[1], "-h") == 0) {
    print_usage(argv[0]);
    exit(0);
  }

  // Initialize device. Note that
  // this function also takes care of registering
  // an deinitialization function using atexit().
  dg100_rd_init(device);

  // Retrieve current configuration
  dg100_get_config(&config);

  while( (opt = getopt(argc, argv, optstring)) != -1 ) {
    switch(opt) {
      case 's':
        if( parse_mode_set_line(optarg, &config) ) {
          update_config = true;
        } else {
          fprintf(stderr, "Error parsing -s option\n");
          exit(-1);
        }
        break;

      case 'p':
        print_config(&config);
        break;

      case 'w':
        if( strcmp(optarg, "enable") == 0 ) {
          config.waas_flag = true;
        } else if( strcmp(optarg, "disable") == 0 ) {
          config.waas_flag = false;
        } else {
          fprintf(stderr, "Invalid argument to -w\n");
          exit(-1);
        }

        update_config = true;
        break;

      case 'x':
        reset_configuration(&config);
        update_config = true;
        break;

      case 'm':
        if( strcmp(optarg, "enable") == 0 ) {
          dg100_set_gpsmouse(true);
        } else if( strcmp(optarg, "disable") == 0 ) {
          dg100_set_gpsmouse(false);
        } else {
          fprintf(stderr, "Invalid argument to -m\n");
          exit(-1);
        }
        break;

      case 't':
        if( strcmp(optarg, "disable") == 0 ) {
          config.spd_thresh_flag = false;
        } else {
          config.spd_thresh_flag = true;
          config.spd_thresh = atoi(optarg) * 100;
        }

        update_config = true;
        break;

      case 'T':
        if( strcmp(optarg, "disable") == 0 ) {
          config.dist_thresh_flag = false;
        } else {
          config.dist_thresh_flag = true;
          config.dist_thresh = atoi(optarg);
        }

        update_config = true;
        break;

      case 'l':
        // Ugly with these blocks, I know
        {
          int mode = atoi(optarg);
          if( mode < 0 || mode > 2) {
            fprintf(stderr, "Invalid argument for -l\n");
            exit(-1);
          }
          config.infotype = mode;
          update_config = true;
        }
        break;

      default:
        print_usage(argv[0]);
        exit(1);
    }
  }

  if( update_config ) {
    fprintf(stderr, "Configuration changed, writing to device...");
    dg100_set_config(&config);
    fprintf(stderr, "Done\n");
  }
}

void print_usage(char *prog_name) {
  printf("Usage: %s [-d DEVICE] [OPTION]\n", prog_name);
  printf("\nDEVICE defaults to /dev/ttyUSB0\n");
  printf("\nPossible values for OPTION are:\n");
  printf("\
  -l LOGGING-MODE     0 = Position Only\n\
                      1 = Position, Speed, Time\n\
                      2 = Position, Speed, Time, Altitude\n\
  -s MODE-NAME,TYPE,VALUE\n\
                      Set the logging mode of MODE-NAME.\n\
                      MODE-NAME can be A, B, or C\n\
                      TYPE can be 'time' or 'distance'\n\
                      VALUE is in an integer in meters for 'distance'\n\
                            and in seconds for 'time'\n\
  -w enable|disable   Enable or disable WAAS/EGNOS/MSAS capabilities\n\
  -t INT|disable      Set speed threshold, in km/h. 'disable' disables the\n\
                      speed threshold.\n\
  -T INT|disable      Set distance threshold, in m. 'disable' disables the\n\
                      distance threshold.\n\
  -p                  Print current configuration.\n\
  -x                  Reset device to default configuration.\n\
                      (All modes time-based, mode A: 30sec, B: 10sec, C: 5sec).\n\
  -m enable|disable   Enable/disable GPS Mouse.\n\
                      (Not very stable at this point.)\n\
  -h                  Print this help screen.\n");
}

void print_config(struct dg100config *config) {
    int i;
    char mode_name[] = {'A','B','C'};
    char *log_info;

    switch(config->infotype) {
      case it_pos:
        log_info = "Position only";
        break;
      case it_pos_speed_time:
        log_info = "Position, speed, and time";
        break;
      case it_pos_speed_time_alt:
        log_info = "Position, speed, time, and altitude";
        break;
    }
    printf("Logging info       : %s\n", log_info);
    printf("Mem free           : %d%%\n", config->memfree);
    printf("WAAS/EGNOS/MSAS    : %s\n", (config->waas_flag == 1 ? "enabled" : "disabled") );
    if( config->spd_thresh_flag == fl_enable ) {
      printf("Speed threshold    : %d km/h\n", config->spd_thresh/100);
    } else {
      printf("Speed threshold    : disabled\n");
    }
    if( config->dist_thresh_flag == fl_enable ) {
      printf("Distance threshold : %d m\n", config->dist_thresh);
    } else {
      printf("Distance threshold : disabled\n");
    }

    printf("\nLogging modes:\n");
    for(i=0; i<3; i++) {
      printf("Mode %c: ", mode_name[i]);
      if( config->loginterval[i].flag == li_time ) {
        printf("by time, every %.2f seconds\n", 
            config->loginterval[i].time/1000.0);
      } else {
        printf("by distance, every %.2f meters\n", 
            config->loginterval[i].dist/1000.0);
      }
    }
}

bool parse_mode_set_line(char *arg, struct dg100config *config)
{
  int modeNameInt;
  int mode;
  int value;

  char *modeName;
  int modeNameLen = -1;

  char *modeType;
  int modeTypeLen = -1;

  char *modeValue;
  int modeValueLen = -1;

  int argLen = strlen(arg);

  // A mode-set line is assumed to be:
  // MODE-NAME,TYPE,VALUE
  // So, we need to locate the two commas to start with

  modeName = arg;
  modeType = index(arg, (char)',');
  if( modeType == NULL )
    return false;

  modeType++;
  // Ensure that we do not read/write beyond the allocated
  // memory
  if( modeType > arg+argLen )
    return false;
  modeNameLen = modeType-modeName-1;

  modeValue = index(modeType, (char)',');
  if( modeValue == NULL )
    return false;

  modeValue++;
  // Ensure that we do not read/write beyond the allocated
  // memory
  if( modeValue > arg+argLen )
    return false;

  modeTypeLen = modeValue-modeType-2;

  // These lines modify the content of arg, which is probably
  // not very nice. But it's easy and we don't need the content
  // at any later point anyhow.
  modeName[modeNameLen] = '\0';
  modeType[modeNameLen+modeTypeLen] = '\0';

  //printf("Mode Name: %s\nMode Type: %s\nMode Value: %s\n", modeName, modeType, modeValue);

  // Only use the first letter of mode name, somewhat undocumented behaviour :-)
  if( !isalpha(modeName[0]) )
    return false;

  modeName[0] = toupper(modeName[0]);

  modeNameInt = modeName[0] - 'A';

  if( modeNameInt > 2 || modeNameInt < 0 )
    return false;

  if( strcmp(modeType, "time") == 0 ) {
    mode = li_time;
  } else if( strcmp(modeType, "distance") == 0) {
    mode = li_distance;
  } else {
    return false;
  }

  value = atoi(modeValue);
  
  config->loginterval[modeNameInt].flag = mode;

  if( mode == li_time ) 
    config->loginterval[modeNameInt].time = value*1000;
  else if( mode == li_distance )
    config->loginterval[modeNameInt].dist = value*1000;
  
  return true;
}

void reset_configuration(struct dg100config *config) {
    config->infotype = it_pos_speed_time_alt;

    config->spd_thresh_flag = fl_disable;
    config->spd_thresh = 0;

    config->dist_thresh_flag = fl_disable;
    config->dist_thresh = 0;

    config->loginterval[0].flag = li_time;
    config->loginterval[0].time = 30 * 1000;
    config->loginterval[0].dist = 0;

    config->loginterval[1].flag = li_time;
    config->loginterval[1].time = 10 * 1000;
    config->loginterval[1].dist = 0;

    config->loginterval[2].flag = li_time;
    config->loginterval[2].time = 5 * 1000;
    config->loginterval[2].dist = 0;

    config->waas_flag = 0;
}
