#include "DG100Dialog.hpp"
#include <QIntValidator>
#include <QSettings>
#include <QMessageBox>
#include <QLabel>
#include <iostream>

DG100Dialog::DG100Dialog(QDialog *confDialog, QWidget *parent)
  : QMainWindow(parent),
    confDialog(confDialog)
{
  setupUi(this);

  statusLabel = new QLabel(this);
  statusLabel->setText( tr("No communication with device") );

  statusBar()->setSizeGripEnabled(false);
  statusBar()->addPermanentWidget(statusLabel);

  QValidator *intValidator = new QIntValidator(this);
  speedThresholdEdit->setValidator( intValidator );
  distThresholdEdit->setValidator( intValidator );

  // Setup connections
  connect(getSettingsButton, SIGNAL(clicked()), this, SLOT(getSettings()));
  connect(setSettingsButton, SIGNAL(clicked()), this, SLOT(setSettings()));
  connect(configureButton, SIGNAL(clicked()), confDialog, SLOT(show()));
  connect(exitButton, SIGNAL(clicked()), QApplication::instance(), SLOT(quit()));
  connect(confDialog, SIGNAL(accepted()), this, SLOT(loadConfiguration()));

  QSettings settings;
  if( !settings.contains("device") ) {
    confDialog->show();
  } else {
    loadConfiguration();
  }
}

DG100Dialog::~DG100Dialog()
{
  delete intValidator;
  delete statusLabel;
}

void
DG100Dialog::getSettings()
{
  QByteArray dev = this->deviceName.toAscii();
  const char *deviceName = dev.constData();

  statusLabel->setText( tr("Retrieving settings from device") );
  centralWidget()->setEnabled( false );

  // Ensure that the GUI is updated before we give control
  // to the dg100-comm code.
  QApplication::instance()->sendPostedEvents();
  QApplication::instance()->processEvents();

  if( !dg100_rd_init(deviceName) ) 
  {
    QMessageBox msgBox(this);
    msgBox.setText(tr("Cannot open device: %1").arg(this->deviceName));
    msgBox.exec();

    centralWidget()->setEnabled( true );
    statusLabel->setText( tr("No communication with device") );

    confDialog->show();
    return;
  }

  dg100_get_config(&deviceConfig);

  loggingModeComboBox->setCurrentIndex(deviceConfig.infotype);
  waasCheckBox->setChecked( (deviceConfig.waas_flag == 1) );

  speedThresholdGroup->setChecked( (deviceConfig.spd_thresh_flag == fl_enable) );
  speedThresholdEdit->setText( QString("%1").arg(deviceConfig.spd_thresh/100) );

  distThresholdGroup->setChecked( (deviceConfig.dist_thresh_flag == fl_enable) );
  distThresholdEdit->setText( QString("%1").arg(deviceConfig.dist_thresh) );

  uiUpdateMode( modeAByTime, modeATimeEdit, 
                modeAByDist, modeADistEdit,
                &(deviceConfig.loginterval[0]) );

  uiUpdateMode( modeBByTime, modeBTimeEdit, 
                modeBByDist, modeBDistEdit,
                &(deviceConfig.loginterval[1]) );

  uiUpdateMode( modeCByTime, modeCTimeEdit, 
                modeCByDist, modeCDistEdit,
                &(deviceConfig.loginterval[2]) );

  // As the device-name might change before we need it next time, we can as well close the
  // connection now
  dg100_rd_deinit();
  centralWidget()->setEnabled( true );
  statusLabel->setText( tr("No communication with device") );
}

void
DG100Dialog::setSettings()
{
  QByteArray dev = this->deviceName.toAscii();
  const char *deviceName = dev.constData();

  statusLabel->setText( tr("Saving settings to device") );
  centralWidget()->setEnabled( false );

  // Ensure that the GUI is updated before we give control
  // to the dg100-comm code.
  QApplication::instance()->sendPostedEvents();
  QApplication::instance()->processEvents();

  if( !dg100_rd_init(deviceName) ) 
  {
    QMessageBox msgBox(this);
    msgBox.setText(tr("Cannot open device: %1").arg(this->deviceName));
    msgBox.exec();

    centralWidget()->setEnabled( true );
    statusLabel->setText( tr("No communication with device") );

    confDialog->show();
    return;
  }

  deviceConfig.infotype = (enum infotype)loggingModeComboBox->currentIndex();

  if( speedThresholdGroup->isChecked() )
    deviceConfig.spd_thresh_flag = fl_enable;
  else
    deviceConfig.spd_thresh_flag = fl_disable;

  if( distThresholdGroup->isChecked() )
    deviceConfig.dist_thresh_flag = fl_enable;
  else
    deviceConfig.dist_thresh_flag = fl_disable;

  deviceConfig.spd_thresh = speedThresholdEdit->text().toInt()*100;
  deviceConfig.dist_thresh = distThresholdEdit->text().toInt();

  if( waasCheckBox->isChecked() )
    deviceConfig.waas_flag = 1;
  else
    deviceConfig.waas_flag = 0;

  updateConfig( &(deviceConfig.loginterval[0]),
                modeAByTime, modeATimeEdit,
                modeAByDist, modeADistEdit );

  updateConfig( &(deviceConfig.loginterval[1]),
                modeBByTime, modeBTimeEdit,
                modeBByDist, modeBDistEdit );

  updateConfig( &(deviceConfig.loginterval[2]),
                modeCByTime, modeCTimeEdit,
                modeCByDist, modeCDistEdit );

  dg100_set_config(&deviceConfig);

  dg100_rd_deinit();

  centralWidget()->setEnabled( true );
  statusLabel->setText( tr("No communication with device") );
}

void 
DG100Dialog::uiUpdateMode(QRadioButton *timeCheckBox, QLineEdit *timeEdit,
                          QRadioButton *distCheckBox, QLineEdit *distEdit,
                          struct loginterval *interval)
{
  timeCheckBox->setChecked( (interval->flag == 0) );
  distCheckBox->setChecked( (interval->flag == 1 ) );

  timeEdit->setText( QString("%1").arg(interval->time/1000) );
  distEdit->setText( QString("%1").arg(interval->dist/1000) );
}

void
DG100Dialog::updateConfig(struct loginterval *interval,
                  QRadioButton *timeButton, QLineEdit *timeEdit,
                  QRadioButton *distButton, QLineEdit *distEdit)
{
  if( timeButton->isChecked() )
    interval->flag = li_time; 
  else if( distButton->isChecked() )
    interval->flag = li_distance;

  interval->time = timeEdit->text().toInt()*1000;
  interval->dist = distEdit->text().toInt()*1000;
}

void
DG100Dialog::loadConfiguration()
{
  QSettings settings;

  QVariant deviceSetting = settings.value("device");
  if( deviceSetting.type() == QVariant::String )
  {
    deviceName = deviceSetting.toString();
    getSettings();
  } else {
    confDialog->show();
  }
}

